package com.mdd.admin.config.quartz.exceution;

import com.mdd.admin.config.quartz.TaskConstants;
import com.mdd.common.entity.Crontab;
import com.mdd.common.mapper.CrontabMapper;
import com.mdd.common.util.SpringUtils;
import com.mdd.common.util.StringUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

public abstract class AbstractQuartzJob implements Job {
   private static final Logger log = LoggerFactory.getLogger(AbstractQuartzJob.class);
   private static final ThreadLocal<Long> threadLocal = new ThreadLocal();

   public void execute(JobExecutionContext context) {
      Crontab crontab = new Crontab();
      BeanUtils.copyProperties(context.getMergedJobDataMap().get("TASK_PROPERTIES"), crontab);

      try {
         this.before();
         this.doExecute(context, crontab);
         this.after(crontab, (Exception)null);
      } catch (Exception var4) {
         Exception e = var4;
         log.error("任务执行异常：", e);
         this.after(crontab, e);
      }

   }

   protected void before() {
      threadLocal.set(System.currentTimeMillis());
   }

   protected void after(Crontab crontab, Exception e) {
      long startTime = (Long)threadLocal.get();
      long endTime = System.currentTimeMillis();
      threadLocal.remove();
      crontab.setError("");
      crontab.setStartTime(startTime / 1000L);
      crontab.setEndTime(endTime / 1000L);
      crontab.setTaskTime(endTime - startTime);
      if (StringUtils.isNotNull(e)) {
         crontab.setError(e.getMessage());
         crontab.setStatus(TaskConstants.STATUS_FAIL);
      }

      ((CrontabMapper)SpringUtils.getBean(CrontabMapper.class)).updateById(crontab);
   }

   protected abstract void doExecute(JobExecutionContext var1, Crontab var2) throws Exception;
}
